/**
 * unit tests for the geis_instance_table module
 */
#include <check.h>

#include "libutouch-geis-xcb/geis_instance.h"
#include "libutouch-geis-xcb/geis_instance_table.h"

/* fixtures */
static GeisInstanceTable fixture_table;
static GeisInstance      fixture_instance;
static const uint32_t    fixture_window_id = 0x32;

/* fixture setup */
static void
construct_table()
{
  fixture_table = geis_instance_table_new(1);
  fixture_instance = geis_instance_new(fixture_window_id);
}

/* fixture teardown */
static void
destroy_table()
{
  geis_instance_free(fixture_instance);
  geis_instance_table_free(fixture_table);
}

/* verify table construction/destruction */
START_TEST(construction)
{
  GeisInstanceTable table = geis_instance_table_new(1);
  fail_unless(table != NULL, "failed to create instance_table");
  geis_instance_table_free(table);
}
END_TEST


/* verify table insertion */
START_TEST(insertion)
{
  geis_instance_table_add(fixture_table, fixture_instance);
}
END_TEST


/* verify table find operation (positive results) */
START_TEST(find_success)
{
  geis_instance_table_add(fixture_table, fixture_instance);
  GeisInstance instance = geis_instance_table_get(fixture_table, fixture_window_id);
  fail_if(instance == NULL, "expected instance not found");
  fail_unless(geis_instance_get_window_id(instance) == fixture_window_id);
}
END_TEST


/* verify table find operation (negative results) */
START_TEST(find_fail)
{
  GeisInstance instance = geis_instance_table_get(fixture_table, 0x5a);
  fail_unless(instance == NULL, "unexpected instance found");
}
END_TEST


/* boilerplate */
Suite *
make_instance_table_suite()
{
  Suite *s = suite_create("geis_instance_table");

  TCase *create = tcase_create("table-creation");
  tcase_add_test(create, construction);
  suite_add_tcase(s, create);

  TCase *usage = tcase_create("table-usage");
  tcase_add_checked_fixture(usage, construct_table, destroy_table);
  tcase_add_test(usage, insertion);
  tcase_add_test(usage, find_success);
  tcase_add_test(usage, find_fail);
  suite_add_tcase(s, usage);

  return s;
}

